package com.winhc.bigdata.spark.ng.dynamic

import com.winhc.bigdata.spark.utils.BaseUtil.{getYesterday, isWindows}
import com.winhc.bigdata.spark.utils.{BaseUtil, LoggingUtils, SparkUtils}
import org.apache.commons.lang3.StringUtils
import org.apache.spark.internal.Logging
import org.apache.spark.rdd.RDD
import org.apache.spark.sql.SparkSession

import scala.annotation.meta.getter
import scala.collection.mutable

/**
 * @author: XuJiakai
 * @date: 2021/6/22 09:13
 */
case class NgCompanyDynamic(s: SparkSession,
                            args: Seq[NgCompanyDynamicArgs]
                            , agg: Seq[NgAcrossTabAggArgs]
                           ) extends LoggingUtils with Logging {
  @(transient@getter) val spark: SparkSession = s

  private val target_tab = "winhc_ng.out_company_dynamic"

  private val args_map: Map[String, NgCompanyDynamicArgs] = args.map(r => (r.tn, r)).toMap

  def init(): Unit = {
    sql(
      s"""
         |CREATE TABLE IF NOT EXISTS $target_tab
         |(
         |    id STRING COMMENT '企业动态主键'
         |    ,association_entity_info STRING COMMENT '关联主体信息,json格式。[{keyno:xxx,name:xxx}]'
         |    ,rowkey STRING COMMENT '企业动态涉及到的rowkey,如果是聚合类型的数据此项为空'
         |    ,tn STRING COMMENT 'rowkey字段对应的tn,如果是聚合类型的数据此项可能为空'
         |    ,update_type STRING COMMENT '动态类型：insert，update，deleted，remove'
         |    ,risk_level STRING COMMENT '变更风险等级'
         |    ,risk_level_detail STRING COMMENT '变更风险等级'
         |    ,dynamic_info STRING COMMENT '动态展示层的相关数据，json格式'
         |    ,agg_detail_text STRING COMMENT '聚合类型的json数据，规范优先'
         |    ,agg_detail_rowkey STRING COMMENT '聚合类型rowkey的多项rowkey字段，结构：tn@@rowkey,tn@@rowkey'
         |    ,old_record STRING COMMENT '上一个版本数据json格式'
         |    ,new_record STRING COMMENT '当前版本数据json格式'
         |    ,change_time string COMMENT '变更时间（业务展示 yyyy-MM-dd）'
         |    ,update_time STRING  COMMENT  '更新时间'
         |    ,create_time STRING COMMENT '创建时间'
         |)
         |COMMENT '企业动态输出表'
         |PARTITIONED BY
         |(
         |    ds STRING COMMENT '分区'
         |)
         |""".stripMargin)

    val i = agg.flatMap(_.tabs).map((_, 1)).groupBy(_._1)
      .mapValues(_.foldLeft(0)(_ + _._2)).count(_._2 >= 2)
    if (i > 0) {
      throw new RuntimeException("agg.tn 有交叉")
    }
  }

  def calc(): Unit = {
    val ds = getYesterday()
    val where = args_map.keys.map(r => s""" "$r" """).mkString("(", ",", ")")

    val rdd: RDD[CompanyDynamicRecord] = sql(
      s"""
         |SELECT  *
         |FROM    winhc_ng.bds_change_extract
         |--- FROM    winhc_ng.bds_change_extract_test
         |WHERE   ds = $ds
         |AND    tn in $where
         |""".stripMargin).rdd.map(r => {
      val value = r.getAs[String]("change_fields")
      val change_fields: Seq[String] = if (StringUtils.isEmpty(value)) Seq.empty else value.split(",")
      ChangeExtract(rowkey = r.getAs("rowkey")
        , company_id = r.getAs("company_id")
        , company_name = null
        , tn = r.getAs("table_name")
        , update_type = r.getAs("update_type")
        , old_data = r.getAs("old_data")
        , new_data = r.getAs("new_data")
        , change_fields = change_fields
        , biz_date = r.getAs("biz_date")
        , update_time = r.getAs("update_time")
      )
    }).filter(r => {
      val filter = args_map(r.tn).filter
      if (filter == null) {
        true
      } else {
        filter.apply(r.update_type, r.biz_date, r.change_fields, r.old_data, r.new_data)
      }
    }).flatMap(r => args_map(r.tn).flat_map.apply(r))
      .map(_.format())
      .filter(_ != null)
      .cache()

    //todo  可将rdd直接落hbase库


    var rdd_map: mutable.Map[String, RDD[CompanyDynamicRecord]] = mutable.Map.empty

    for (elem <- args) {
      var tmp_rdd: RDD[CompanyDynamicRecord] = null
      if (elem.group_by_key == null) {
        tmp_rdd = rdd.filter(r => elem.tn.equals(r.tn))
      } else {
        if (elem.group_by_pre == null) {
          tmp_rdd = rdd.filter(r => elem.tn.equals(r.tn))
            .groupBy(r => args_map(elem.tn).group_by_key.apply(r))
            .flatMap(r => args_map(elem.tn).group_by_flat_map(r._2.toSeq))
        } else {
          tmp_rdd = rdd.filter(r => elem.tn.equals(r.tn))
            .flatMap(r => args_map(elem.tn).group_by_pre.apply(r))
            .filter(_ != null)
            .groupBy(r => args_map(elem.tn).group_by_key.apply(r))
            .flatMap(r => args_map(elem.tn).group_by_flat_map(r._2.toSeq))
        }
      }
      rdd_map = rdd_map + (elem.tn -> tmp_rdd)
    }

    for (elem <- agg) {
      var tmp_rdd: RDD[CompanyDynamicRecord] = null
      for (tn <- elem.tabs) {
        if (tmp_rdd == null) {
          rdd_map(tn)
        } else {
          tmp_rdd = tmp_rdd.union(rdd_map(tn))
        }
        rdd_map = rdd_map - tn
      }
      if (elem.group_by_pre == null) {
        tmp_rdd = tmp_rdd.groupBy(elem.group_by_key).flatMap(r => elem.group_by_flat_map.apply(r._2.toSeq))
      } else {
        tmp_rdd = tmp_rdd
          .flatMap(r => elem.group_by_pre.apply(r))
          .groupBy(elem.group_by_key).flatMap(r => elem.group_by_flat_map.apply(r._2.toSeq))
      }
      rdd_map = rdd_map + (elem.tabs.mkString("_") -> tmp_rdd)
    }

    var out_rdd: RDD[CompanyDynamicRecord] = null

    for ((key, value) <- rdd_map) {
      if (out_rdd == null) {
        out_rdd = value
      } else {
        out_rdd = out_rdd.union(value)
      }
    }

    spark.createDataFrame(out_rdd.map(_.to_row()), spark.table(target_tab).schema)
      .createTempView("company_dynamic_out_tab")

    sql(
      s"""
         |INSERT ${if (isWindows) "INTO" else "OVERWRITE"}  TABLE $target_tab PARTITION(ds='${BaseUtil.getYesterday()}')
         |SELECT id
         |       ,association_entity_info
         |       ,rowkey
         |       ,tn
         |       ,update_type
         |       ,risk_level
         |       ,risk_level_detail
         |       ,dynamic_info
         |       ,agg_detail_text
         |       ,agg_detail_rowkey
         |       ,old_record
         |       ,new_record
         |       ,change_time
         |       ,update_time
         |       ,create_time
         |FROM   company_dynamic_out_tab
         |""".stripMargin)
  }
}

object NgCompanyDynamic {

  def main(args: Array[String]): Unit = {
    val config = mutable.Map(
      "spark.hadoop.odps.project.name" -> "winhc_ng",
      "spark.debug.maxToStringFields" -> "200",
      "spark.hadoop.odps.spark.local.partition.amt" -> "1000000"
    )
    val spark = SparkUtils.InitEnv(this.getClass.getSimpleName, config)

    NgCompanyDynamic(spark, NgCompanyDynamicArgs.getStartArgs, NgCompanyDynamicArgs.getAggArgs).calc()
    spark.stop()
  }

}
